'use strict';

var _defineProperty2 = require('babel-runtime/helpers/defineProperty');

var _defineProperty3 = _interopRequireDefault(_defineProperty2);

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _slicedToArray2 = require('babel-runtime/helpers/slicedToArray');

var _slicedToArray3 = _interopRequireDefault(_slicedToArray2);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _expect = require('expect');

var _expect2 = _interopRequireDefault(_expect);

var _graphCrdt = require('graph-crdt');

var _index = require('../../index');

var _index2 = _interopRequireDefault(_index);

var _mocks = require('../../mocks');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/* eslint-env mocha */
describe('Storage plugin', function () {
  var db = void 0,
      storage = void 0;

  beforeEach(function () {
    storage = new _mocks.Storage();

    db = (0, _index2.default)({
      storage: [storage]
    });
  });

  describe('write', function () {
    var write = void 0;

    beforeEach(function () {
      write = (0, _expect.spyOn)(storage, 'write').andCallThrough();
    });

    it('is given a graph', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee() {
      var _write$calls$0$argume, graph;

      return _regenerator2.default.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              _context.next = 2;
              return db.write('users', {});

            case 2:

              (0, _expect2.default)(write).toHaveBeenCalled();
              _write$calls$0$argume = (0, _slicedToArray3.default)(write.calls[0].arguments, 1), graph = _write$calls$0$argume[0].graph;

              (0, _expect2.default)(graph).toBeA(_graphCrdt.Graph);

            case 5:
            case 'end':
              return _context.stop();
          }
        }
      }, _callee, undefined);
    })));

    it('contains complete node state', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee2() {
      var _write$calls$1$argume, graph, node;

      return _regenerator2.default.wrap(function _callee2$(_context2) {
        while (1) {
          switch (_context2.prev = _context2.next) {
            case 0:
              _context2.next = 2;
              return db.write('state', { original: true });

            case 2:
              _context2.next = 4;
              return db.write('state', { update: true });

            case 4:

              // Assert full in-memory state.
              (0, _expect2.default)(write.calls.length).toBe(2);

              _write$calls$1$argume = (0, _slicedToArray3.default)(write.calls[1].arguments, 1), graph = _write$calls$1$argume[0].graph;
              node = graph.value('state');


              (0, _expect2.default)([].concat((0, _toConsumableArray3.default)(node))).toEqual([['original', true], ['update', true]]);

            case 8:
            case 'end':
              return _context2.stop();
          }
        }
      }, _callee2, undefined);
    })));

    it('is passed node updates', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee3() {
      var profile;
      return _regenerator2.default.wrap(function _callee3$(_context3) {
        while (1) {
          switch (_context3.prev = _context3.next) {
            case 0:
              _context3.next = 2;
              return db.write('profile', {
                name: 'Bob Carlson VII'
              });

            case 2:
              profile = _context3.sent;


              (0, _expect2.default)(write.calls.length).toBe(1);
              _context3.next = 6;
              return profile.write('username', 'SuperBob');

            case 6:
              (0, _expect2.default)(write.calls.length).toBe(2);

            case 7:
            case 'end':
              return _context3.stop();
          }
        }
      }, _callee3, undefined);
    })));

    it('is passed the options object', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee4() {
      var _write$calls$0$argume2, action;

      return _regenerator2.default.wrap(function _callee4$(_context4) {
        while (1) {
          switch (_context4.prev = _context4.next) {
            case 0:
              _context4.next = 2;
              return db.write('users', {}, {
                potatoes: true
              });

            case 2:
              _write$calls$0$argume2 = (0, _slicedToArray3.default)(write.calls[0].arguments, 1), action = _write$calls$0$argume2[0];

              (0, _expect2.default)(action).toContain({
                potatoes: true
              });

            case 4:
            case 'end':
              return _context4.stop();
          }
        }
      }, _callee4, undefined);
    })));

    it('is not called if storage is disabled', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee5() {
      return _regenerator2.default.wrap(function _callee5$(_context5) {
        while (1) {
          switch (_context5.prev = _context5.next) {
            case 0:
              _context5.next = 2;
              return db.write('users', {}, {
                storage: []
              });

            case 2:
              (0, _expect2.default)(write).toNotHaveBeenCalled();

            case 3:
            case 'end':
              return _context5.stop();
          }
        }
      }, _callee5, undefined);
    })));
  });

  describe('read', function () {
    var read = void 0,
        graph = void 0,
        node = void 0;

    beforeEach(function () {
      read = (0, _expect.spyOn)(storage, 'read').andCallThrough();

      graph = new _graphCrdt.Graph();
      node = new _graphCrdt.Node();
    });

    it('is used to retrieve state', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee6() {
      var result;
      return _regenerator2.default.wrap(function _callee6$(_context6) {
        while (1) {
          switch (_context6.prev = _context6.next) {
            case 0:
              node.merge({ from: 'storage' });
              graph.merge((0, _defineProperty3.default)({}, node, node));

              _context6.next = 4;
              return storage.write({ graph: graph });

            case 4:
              _context6.next = 6;
              return db.read(node.toString());

            case 6:
              result = _context6.sent;

              (0, _expect2.default)(read).toHaveBeenCalled();
              (0, _expect2.default)(result).toBeA(_graphCrdt.Node);

              (0, _expect2.default)([].concat((0, _toConsumableArray3.default)(result))).toEqual([['from', 'storage']]);

            case 10:
            case 'end':
              return _context6.stop();
          }
        }
      }, _callee6, undefined);
    })));

    it('is not called if the value is cached', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee7() {
      return _regenerator2.default.wrap(function _callee7$(_context7) {
        while (1) {
          switch (_context7.prev = _context7.next) {
            case 0:
              _context7.next = 2;
              return db.write('stuff', { value: 'probably' });

            case 2:
              _context7.next = 4;
              return db.read('stuff');

            case 4:
              (0, _expect2.default)(read).toNotHaveBeenCalled();

            case 5:
            case 'end':
              return _context7.stop();
          }
        }
      }, _callee7, undefined);
    })));

    it('caches read results', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee8() {
      return _regenerator2.default.wrap(function _callee8$(_context8) {
        while (1) {
          switch (_context8.prev = _context8.next) {
            case 0:
              node.merge({ from: 'storage' });
              graph.merge((0, _defineProperty3.default)({}, node, node));
              _context8.next = 4;
              return storage.write({ graph: graph });

            case 4:

              (0, _expect2.default)(read.calls.length).toBe(0);

              // First read.
              _context8.next = 7;
              return db.read(node.toString());

            case 7:
              (0, _expect2.default)(read.calls.length).toBe(1);

              // Second read.
              _context8.next = 10;
              return db.read(node.toString());

            case 10:
              (0, _expect2.default)(read.calls.length).toBe(1);

            case 11:
            case 'end':
              return _context8.stop();
          }
        }
      }, _callee8, undefined);
    })));

    it('is passed the options object', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee9() {
      var _read$calls$0$argumen, options;

      return _regenerator2.default.wrap(function _callee9$(_context9) {
        while (1) {
          switch (_context9.prev = _context9.next) {
            case 0:
              _context9.next = 2;
              return db.read('key', { potatoes: true });

            case 2:
              _read$calls$0$argumen = (0, _slicedToArray3.default)(read.calls[0].arguments, 1), options = _read$calls$0$argumen[0];

              (0, _expect2.default)(options).toBeAn(Object);
              (0, _expect2.default)(options).toContain({ potatoes: true });

            case 5:
            case 'end':
              return _context9.stop();
          }
        }
      }, _callee9, undefined);
    })));

    it('is not called if storage is disabled', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee10() {
      return _regenerator2.default.wrap(function _callee10$(_context10) {
        while (1) {
          switch (_context10.prev = _context10.next) {
            case 0:
              _context10.next = 2;
              return db.read('key', { storage: [] });

            case 2:
              (0, _expect2.default)(read).toNotHaveBeenCalled();

            case 3:
            case 'end':
              return _context10.stop();
          }
        }
      }, _callee10, undefined);
    })));

    it('gives the same context as if cached', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee11() {
      var node, storage, db, _node$meta, uid;

      return _regenerator2.default.wrap(function _callee11$(_context11) {
        while (1) {
          switch (_context11.prev = _context11.next) {
            case 0:
              node = _graphCrdt.Node.from({ data: true });
              storage = new _mocks.Storage();

              storage.read = (0, _expect.createSpy)();
              storage.read.andReturn(node.toJSON());

              db = (0, _index2.default)({ storage: [storage] });
              _node$meta = node.meta(), uid = _node$meta.uid;
              _context11.next = 8;
              return db.read(uid);

            case 8:
              _context11.t1 = _context11.sent;
              _context11.t0 = (0, _expect2.default)(_context11.t1);
              _context11.next = 12;
              return db.read(uid);

            case 12:
              _context11.t2 = _context11.sent;

              _context11.t0.toBe.call(_context11.t0, _context11.t2);

            case 14:
            case 'end':
              return _context11.stop();
          }
        }
      }, _callee11, undefined);
    })));
  });
});