'use strict';

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var _index = require('./index');

var _index2 = _interopRequireDefault(_index);

var _expect = require('expect');

var _expect2 = _interopRequireDefault(_expect);

var _index3 = require('../connection-group/index');

var _index4 = _interopRequireDefault(_index3);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

describe('A config', function () {
  var result = void 0;

  beforeEach(function () {
    result = (0, _index2.default)([]);
  });

  it('contains storage settings', function () {
    (0, _expect2.default)(result.storage).toEqual([]);
  });

  it('contains query engines', function () {
    (0, _expect2.default)(result.engines).toEqual({});
  });

  it('contains the router', function () {
    (0, _expect2.default)(result.router).toBe(null);
  });

  it('contains an extend object', function () {
    (0, _expect2.default)(result.extend).toEqual({
      root: {},
      context: {}
    });
  });

  it('contains network settings', function () {
    (0, _expect2.default)(result.network).toBeA(_index4.default);
  });

  it('contains a hooks object', function () {
    var hook = {
      read: {
        node: [],
        field: []
      },
      write: [],
      request: [],
      update: []
    };
    (0, _expect2.default)(result.hooks).toEqual({
      before: hook,
      after: hook,
      catch: hook
    });
  });

  describe('merge', function () {
    it('does not add undefined items', function () {
      var _config = (0, _index2.default)([{}]),
          hooks = _config.hooks,
          network = _config.network,
          storage = _config.storage;

      (0, _expect2.default)(storage.length).toBe(0);
      (0, _expect2.default)([].concat((0, _toConsumableArray3.default)(network)).length).toBe(0);
      (0, _expect2.default)(hooks.before.read.node.length).toBe(0);
      (0, _expect2.default)(hooks.before.read.field.length).toBe(0);
      (0, _expect2.default)(hooks.before.write.length).toBe(0);
      (0, _expect2.default)(hooks.before.request.length).toBe(0);
      (0, _expect2.default)(hooks.before.update.length).toBe(0);
    });

    it('adds extensions', function () {
      var noop = function noop() {};
      result = (0, _index2.default)([{
        extend: {
          root: { noop: noop },
          context: { noop: noop }
        }
      }]);

      (0, _expect2.default)(result.extend.root).toContain({ noop: noop });
      (0, _expect2.default)(result.extend.context).toContain({ noop: noop });
    });

    it('does not mutate the base "extend"', function () {
      (0, _index2.default)([{
        extend: {
          root: { root: true },
          context: { context: true }
        }
      }]);

      (0, _expect2.default)(_index.base.extend.root).toEqual({});
      (0, _expect2.default)(_index.base.extend.context).toEqual({});
    });

    it('adds storage', function () {
      var storage = { name: 'Storage driver' };

      var result = (0, _index2.default)([{
        storage: [storage]
      }]);

      (0, _expect2.default)(result.storage).toEqual([storage]);
    });

    it('adds query engines', function () {
      var engine = { version: 'potato9000' };

      var engines = {
        potatoQL: engine
      };

      var result = (0, _index2.default)([{ engines: engines }]);
      (0, _expect2.default)(result.engines).toEqual({
        potatoQL: engine
      });

      // Should not mutate base.
      (0, _expect2.default)(_index.base.engines).toEqual({});
    });

    it('merges before hooks', function () {
      var write = function write() {};
      var hooks = {
        before: { write: write }
      };

      var result = (0, _index2.default)([{ hooks: hooks }]);

      (0, _expect2.default)(result.hooks.before.write).toEqual([write]);
    });

    it('adds storage drivers in the order they\'re defined', function () {
      var first = function first() {};
      var second = function second() {};

      var result = (0, _index2.default)([{
        storage: [first]
      }, {
        storage: [second]
      }]);

      (0, _expect2.default)(result.storage).toEqual([first, second]);
    });

    it('adds hooks in the order they\'re defined', function () {
      var first = function first() {};
      var second = function second() {};

      var result = (0, _index2.default)([{
        hooks: {
          before: {
            read: { node: first }
          }
        }
      }, {
        hooks: {
          before: {
            read: { node: second }
          }
        }
      }]);

      (0, _expect2.default)(result.hooks.before.read.node).toEqual([first, second]);
    });

    it('prefers later-defined extensions when conflicts happen', function () {
      var first = function first() {};
      var second = function second() {};

      var result = (0, _index2.default)([{
        extend: {
          root: { method: first },
          context: { method: first }
        }
      }, {
        extend: {
          root: { method: second },
          context: { method: second }
        }
      }]);

      (0, _expect2.default)(result.extend.root.method).toBe(second);
      (0, _expect2.default)(result.extend.context.method).toBe(second);
    });
  });

  describe('network merge', function () {
    var group1 = void 0,
        group2 = void 0,
        conn = void 0;

    var Connection = function Connection(_ref) {
      var id = _ref.id;
      (0, _classCallCheck3.default)(this, Connection);
      this.type = 'test-connection';

      this.id = id;
    };

    beforeEach(function () {
      group1 = new _index4.default();
      group2 = new _index4.default();
      conn = new Connection({ id: 'connection' });
    });

    it('includes every client', function () {
      group1.add(conn);

      var result = (0, _index2.default)([{ network: group1 }]);

      (0, _expect2.default)([].concat((0, _toConsumableArray3.default)(result.network))).toContain(conn);
    });

    it('does not unnecessarily wrap groups', function () {
      group1.add(conn);

      var result = (0, _index2.default)([{ network: group1 }]);

      (0, _expect2.default)(result.network).toBe(group1);
    });

    it('combines clients from every group', function () {
      var conn2 = new Connection({ id: 'conn2' });
      group1.add(conn);
      group2.add(conn2);

      var result = (0, _index2.default)([{
        network: group1
      }, {
        network: group2
      }]);

      (0, _expect2.default)([].concat((0, _toConsumableArray3.default)(result.network))).toContain(conn);
      (0, _expect2.default)([].concat((0, _toConsumableArray3.default)(result.network))).toContain(conn2);
    });

    it('converts single connections into connection groups', function () {
      var conn2 = new Connection({ id: 'conn2' });

      var result = (0, _index2.default)([{
        network: conn
      }, {
        network: conn2
      }]);

      (0, _expect2.default)(result.network).toBeA(_index4.default);
      (0, _expect2.default)([].concat((0, _toConsumableArray3.default)(result.network))).toContain(conn);
      (0, _expect2.default)([].concat((0, _toConsumableArray3.default)(result.network))).toContain(conn2);
    });
  });

  describe('router merge', function () {
    it('adds the router', function () {
      var router = { 'me-a-router': 'yey' };
      var result = (0, _index2.default)([{ router: router }]);

      (0, _expect2.default)(result.router).toBe(router);
    });

    it('throws if given more than one router', function () {
      var fail = function fail() {
        return (0, _index2.default)([{
          router: {}
        }, {
          router: {}
        }]);
      };

      (0, _expect2.default)(fail).toThrow(/router/i);
    });
  });
}); /* eslint-disable require-jsdoc */
/* eslint-env mocha */